@@ -177,10 +177,11 @@ void oRollback (struct tBuf *   pBuf)
     pLastBuf = pBuf ;
     }
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// commit output transaction (all the output since corresponding begin is vaild)
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* commit output transaction (all the output since corresponding begin is vaild)*/
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 void oCommit (struct tBuf *   pBuf) 
 
@@ -212,13 +213,14 @@ void oCommit (struct tBuf *   pBuf)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// write to a buffer
-//
-// we will alloc a new buffer for every write
-// this is fast with apache palloc or for malloc if no free is call in between
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* write to a buffer                                                            */
+/*                                                                              */
+/* we will alloc a new buffer for every write                                   */
+/* this is fast with apache palloc or for malloc if no free is call in between  */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 static int bufwrite (/*in*/ const void * ptr, size_t size) 
@@ -256,14 +258,15 @@ static int bufwrite (/*in*/ const void * ptr, size_t size)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// free buffers
-//
-// free all buffers
-// note: this is not nessecary for apache palloc, because all buffers are freed
-//       at the end of the request
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* free buffers                                                                 */
+/*                                                                              */
+/* free all buffers                                                             */
+/* note: this is not nessecary for apache palloc, because all buffers are freed */
+/*       at the end of the request                                              */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 static void buffree ()
@@ -311,10 +314,11 @@ static void buffree ()
     pLastFreeBuf  = NULL ;
     }
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// get the length outputed to buffers so far
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* get the length outputed to buffers so far                                    */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 int GetContentLength ()
     {
@@ -325,10 +329,11 @@ int GetContentLength ()
     
     }
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// set the name of the input file and open it
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* set the name of the input file and open it                                   */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int OpenInput (/*in*/ const char *  sFilename)
@@ -373,10 +378,11 @@ int OpenInput (/*in*/ const char *  sFilename)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// close the input file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* close input file                                                             */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int CloseInput ()
@@ -397,10 +403,11 @@ int CloseInput ()
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// read block of data from input (web client)
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* read block of data from input (web client)                                   */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int iread (/*in*/ void * ptr, size_t size, size_t nmemb) 
@@ -429,10 +436,11 @@ int iread (/*in*/ void * ptr, size_t size, size_t nmemb)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// read line of data from input (web client)
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* read line of data from input (web client)                                    */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 char * igets    (/*in*/ char * s,   int    size) 
@@ -445,20 +453,24 @@ char * igets    (/*in*/ char * s,   int    size)
 
 #ifdef PerlIO
         {
+        /*
         FILE * f = PerlIO_exportFILE (ifd, 0) ;
         char * p = fgets (s, size, f) ;
         PerlIO_releaseFILE (ifd, f) ;
         return p ;
+        */
+        return NULL ;
         }
 #else
     return fgets (s, size, ifd) ;
 #endif
     }
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// read HTML File into PBuf
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* read HTML File into pBuf                                                     */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 int ReadHTML (/*in*/    char *  sInputfile,
               /*in*/    size_t  nFileSize,
@@ -500,10 +512,12 @@ int ReadHTML (/*in*/    char *  sInputfile,
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// set the name of the output file and open it
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* set the name of the output file and open it                                  */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
+
 
 
 int OpenOutput (/*in*/ const char *  sFilename)
@@ -577,10 +591,11 @@ int OpenOutput (/*in*/ const char *  sFilename)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// close the output file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* close the output file                                                        */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int CloseOutput ()
@@ -605,10 +620,12 @@ int CloseOutput ()
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// set output to memory buffer
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* set output to memory buffer                                                  */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
+
 
 
 void OutputToMemBuf (/*in*/ char *  pBuf,
@@ -620,10 +637,11 @@ void OutputToMemBuf (/*in*/ char *  pBuf,
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// set output to stdandard
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* set output to standard                                                       */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 void OutputToStd    ()
@@ -635,12 +653,11 @@ void OutputToStd    ()
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// puts to output (web client)
-*/
-
-
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* puts to output (web client)                                                  */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 int oputs (/*in*/ const char *  str) 
 
@@ -649,11 +666,11 @@ int oputs (/*in*/ const char *  str)
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// write block of data to output (web client)
-*/
-
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* write block of data to output (web client)                                   */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 int owrite (/*in*/ const void * ptr, size_t size, size_t nmemb) 
 
@@ -705,10 +722,11 @@ int owrite (/*in*/ const void * ptr, size_t size, size_t nmemb)
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// write one char to output (web client)
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* write one char to output (web client)                                        */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 void oputc (/*in*/ char c)
@@ -737,10 +755,11 @@ void oputc (/*in*/ char c)
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// set the name of the log file and open it
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* set the name of the log file and open it                                     */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int OpenLog (/*in*/ const char *  sFilename)
@@ -770,10 +789,11 @@ int OpenLog (/*in*/ const char *  sFilename)
     return ok ;
     }
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// return the handle of the log file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* return the handle of the log file                                            */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int GetLogHandle ()
@@ -783,10 +803,11 @@ int GetLogHandle ()
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// return the current posittion of the log file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* return the current posittion of the log file                                 */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 long GetLogFilePos ()
@@ -796,10 +817,11 @@ long GetLogFilePos ()
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// close the log file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* close the log file                                                           */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int CloseLog ()
@@ -815,10 +837,11 @@ int CloseLog ()
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// flush the log file 
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* flush the log file                                                           */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int FlushLog ()
@@ -831,10 +854,11 @@ int FlushLog ()
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// printf to log file
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* printf to log file                                                           */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 int lprintf (/*in*/ const char *  sFormat,
              /*in*/ ...) 
@@ -861,10 +885,11 @@ int lprintf (/*in*/ const char *  sFormat,
     }
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// write block of data to log file
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* write block of data to log file                                              */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 int lwrite (/*in*/ const void * ptr, size_t size, size_t nmemb) 
@@ -883,10 +908,11 @@ int lwrite (/*in*/ const void * ptr, size_t size, size_t nmemb)
 
 
 
-/* ///////////////////////////////////////////////////////////////////////////////////
-//
-// Memory Allocation
-*/
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* Memory Allocation                                                            */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
 
 
 void _free (void * p)
